/*** Libraries ***/

import fullscreen.*; 
import processing.opengl.*;
import peasy.*;

/*** Global objects and variables ***/

PeasyCam                    cam;
FullScreen                  fs;


// Handtracking
HandCapture                 handCapture;
boolean                     drawFlag=false;

ObjectContainer             objectCont;

int                         userKey = 0;

PVector                     hand1Pos = new PVector();
PVector                     hand2Pos = new PVector();

int                         globalType = 0;

float                       globalState = 0;

int                         globalDirection = 1;

boolean                     increment = false;

float                       waitCounter = -1;

/*** Config 
* This is the main section to make changes to. Go to town!
***/

boolean                     kinect = false;

/*** Config: grid ***/

PVector                     workspace = new PVector(1024, 768);

int                         gridAxisSize = 25;

int                         gridSize = (int)sq(gridAxisSize);

//defaultOptions values

color                       defaultFilling = color(0);
color                       defaultStroke = color(255);

color                       defaultStrokeWeight = 1;

int                         defaultInteractionMode = 6;

//defaultOptions values end

int                         mouseMode = 1;                         //Mouse mode 1: Use mouse position for interaction; Mouse mode 2: Use mouse as a value regulator

float                       stateTransition = 0.05;

boolean                     cursor = true;

/*** Setup ***/

ArrayList                   defaultOptions = new ArrayList();

boolean  block = false;

void setup() {

 size((int)workspace.x, (int)workspace.y, OPENGL);

 if(kinect == true) {
   
   fs = new FullScreen(this);
  
   fs.enter();

   handCapture = new HandCapture(this);
 
  }
     
  defaultOptions.add(defaultFilling);
  defaultOptions.add(defaultStroke);
  defaultOptions.add(defaultStrokeWeight);
  defaultOptions.add(defaultInteractionMode);
  
  /*** Setup objects ***/
  
  objectCont = new ObjectContainer(gridAxisSize, gridSize, 0, 0, defaultOptions);
  

  
  /*** Setup style ***/
  
  noStroke();
  noFill();
  
  smooth();
  
  background(255);
  

  /*** Setup cam ***/
  
  cam = new PeasyCam(this, width / 2, workspace.x / 2, 0, 1000);
  cam.setMinimumDistance(50);
  cam.setMaximumDistance(5000);


}



void draw() {
 
  
  if(kinect == false) {
  
    hand1Pos.x = mouseX;
    hand1Pos.y = mouseY;
    hand1Pos.z = 0;
  
  } else {
    
   handCapture.update();
   
   if(handCapture.isHandTracked())
   {  
      pushStyle();
        stroke(255,0,0);
        strokeWeight(8);
        
        PVector handPos = handCapture.handPos();
        //point(handPos.x,handPos.y);
        
        hand1Pos.x = map(handPos.x, 0, handCapture.width(), 0, workspace.x);
        hand1Pos.y = map(handPos.y, 0, handCapture.height(), 0, workspace.y / 4 * 5);
        hand1Pos.z = handPos.z;
        if(hand1Pos.z < 1200) mouseMode = 2;
        else mouseMode = 1;
      popStyle();
    }
    
  }
  
    
  //fill(200, 150);
  
  background(0);
  
  //rect(-2000, -2000, 4000, 4000);
  
  if(userKey != 0) objectCont.setInteractionMode(userKey - 49);
  
  objectCont.morphAll(globalType, globalState);
  
  objectCont.draw();
  
  
  if(increment == true) {
    
    
    if(globalDirection == 1) {
    
      globalState += stateTransition;
        
      if(globalState >= 1) {
        
        globalType++;
        
        globalState = 0;
        
      } 
    
    } else if(globalDirection == -1) {

      globalState = (globalState - stateTransition);
        
      if(globalState <= stateTransition) {
        
        globalType--;
        
        globalState = 1;
        
      } 
      
    }
    
    
    if(globalType >= 10 && globalDirection == 1) globalDirection = -1;
    
    else if (globalType <= -1 && globalDirection == -1) globalDirection = 1; 
    
    
  } else if (increment == false && mouseMode == 2) {
    
    /*X Axis*/
    
    float stateAndType = map(hand1Pos.x, 0, workspace.x, 0, 10);
  
    /*globalType = floor(stateAndType / 10);
  
    globalState = stateAndType % 10;*/
   
    float globalDistance = stateAndType - (globalType + globalState);
   
   
    if(abs(globalDistance) > 0) {
     
      globalState += globalDistance * stateTransition;
      
      if(globalState >= 1) {
        
        globalType = (globalType + 1) % 10;
      
        globalState = 0;
      
      } else if(globalState <= 0) {
        
        globalType = (globalType - 1) % 10;
        
        globalState = 1;
        
      }
      
      
    } 
    
  }
  
  if(mouseMode == 2) {
        
    /*Y Axis*/
    
    /*Switch between interactions by gesture at the top of the screen */
    
    if(waitCounter == -1) waitCounter = millis();
    
    if(hand1Pos.y < workspace.y / 4 && waitCounter + 2000 <= millis()) {
      
      int interactionMode = (objectCont.getInteractionMode() + 1) % 6;
      
      objectCont.setInteractionMode(interactionMode);
      
      waitCounter = millis();
                  
    } else if(hand1Pos.y < workspace.y / 4 && waitCounter + 2000 > millis()) {
      
      stroke(150, 27, 60, abs(waitCounter - millis()) / 4);
      strokeWeight(10);
      
      //ellipse(150, 300, 100, 100);
          
    } else if(hand1Pos.y <= workspace.y && hand1Pos.y >= workspace.y / 4) {
     
      noStroke();
      
      waitCounter = millis();
      
    } else if(hand1Pos.y >= workspace.y / 4) {
      
      noStroke();
      
    }
    
    fill(255, 100);
    
    rect(0, 0, workspace.x, workspace.y / 4);
    
    
    /*Toggle on auto-increment by gesture at the bottom of the screen */
    
    if(hand1Pos.y > workspace.y && hand1Pos.y < workspace.y / 4 * 5 && waitCounter + 2000 <= millis()) {
      
      increment = ! increment;
      
      waitCounter = millis();
      
    } else if(hand1Pos.y > workspace.y && hand1Pos.y < workspace.y / 4 * 5 && waitCounter + 2000 > millis()) {
      
      stroke(150, 27, 60, abs(waitCounter - millis()) / 4);
      strokeWeight(10);
      
      //ellipse(150, workspace.y - 300, 100, 100);
          
    } else if(hand1Pos.y >= workspace.y / 4 && hand1Pos.y <= workspace.y) {
      
      noStroke();
      
      waitCounter = millis();
      
    } else if(hand1Pos.y <= workspace.y) {
      
      noStroke();
      
    }
        
    fill(255, 100);
    
    rect(0, workspace.y, workspace.x, workspace.y / 4); 
    
  }
  
  
  if(mouseMode == 2 || increment == true) {
   
       drawHud();
    
  }
    
  pushMatrix();
  
    fill(200, 27, 60, 250);
  
    translate(hand1Pos.x, hand1Pos.y);
    
    if(cursor == true) ellipse(0, 0, 10, 10);
  
  popMatrix();
  
}



void mouseClicked() {
 
  if(mouseButton == LEFT) {
  
    increment = ! increment;
  
  }
  
}

void mousePressed() {
 
  if(kinect == false) mouseMode = 2;
  
}

void mouseReleased() {
 
 if(kinect == false) mouseMode = 1;
  
}


/*void mouseMoved() {
  
}*/


void drawHud() {
 
  float stateOfTheHud = map(globalType + globalState, 0, 10, 50, workspace.x - 100);
    
  if(stateOfTheHud >= 0) {
  
    pushMatrix();
    
      cam.beginHUD();  
    
        stroke(50, 150);
        strokeWeight(1);
        noFill();
    
        //rect(50, workspace.y - 100, workspace.x - 100, 50);
                
        //for(int i = 1; i < 50; i++) {
          
          fill(150, 27, 60, 150);
          
          //line(50, workspace.y - 100 + i, stateOfTheHud, workspace.y - 100 + i);
          
          rect(50, workspace.y - 50, stateOfTheHud, 50);
                    
        //}
    
      cam.endHUD();
    
    popMatrix();
  
  }
  
}



void keyPressed() {
  
  if(keyCode >= 49 && keyCode <= 59) userKey = keyCode;
  
  if(keyCode == 82) {
   
    cursor = ! cursor;
    
  }
      
 // redraw();

}




// -----------------------------------------------------------------
// hand events

void onCreateHands(int handId,PVector pos,float time)
{
  handCapture.onCreateHands(handId,pos,time);
}

void onUpdateHands(int handId,PVector pos,float time)
{
  handCapture.onUpdateHands(handId,pos,time);
}

void onDestroyHands(int handId,float time)
{
  handCapture.onDestroyHands(handId,time);
}

void onRecognizeGesture(String strGesture, PVector idPosition, PVector endPosition)
{
  handCapture.onRecognizeGesture(strGesture,idPosition,endPosition);
}

void onProgressGesture(String strGesture, PVector position,float progress)
{
  handCapture.onProgressGesture(strGesture,position,progress);
}
